//
//  AQPlayFileController.m
//  AudioQueuePlayFile
//
//  Copyright 2011 JG Electronics. All rights reserved.
//	Remarks, reports to: www.jgelectronics.nl.
//
//	Derived from Apple's example program AudioQueueTools.
//	The application icon is from Vincent Garnier http://benjigarner.deviantart.com/art/Fruity-Apples-80805804
//
//	OSX10.6 and XCODE 4.0.1
//
/*
 Disclaimer: IMPORTANT: 
 This JG Electronics software is supplied to you by JG Electronics as FREE AND OPEN SOURCE SOFTWARE. 
 
 The JG Electronics Software is provided by JG Electronics on an "AS IS" basis.
 JG ELECTRONICS MAKES NO WARRANTIES, EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION
 THE IMPLIED WARRANTIES OF NON-INFRINGEMENT, MERCHANTABILITY AND FITNESS
 FOR A PARTICULAR PURPOSE, REGARDING THE JG ELECTRONICS SOFTWARE OR ITS USE AND
 OPERATION ALONE OR IN COMBINATION WITH YOUR PRODUCTS.
 
 IN NO EVENT SHALL JG ELECTRONICS BE LIABLE FOR ANY SPECIAL, INDIRECT, INCIDENTAL
 OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
 SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 INTERRUPTION) ARISING IN ANY WAY OUT OF THE USE, REPRODUCTION,
 MODIFICATION AND/OR DISTRIBUTION OF THE JG ELECTRONICS SOFTWARE, HOWEVER CAUSED
 AND WHETHER UNDER THEORY OF CONTRACT, TORT (INCLUDING NEGLIGENCE),
 STRICT LIABILITY OR OTHERWISE, EVEN IF JG ELECTRONICS HAS BEEN ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.
 */

#import "AQPlayFileController.h"

@implementation AQPlayFileController

	#pragma mark -
	#pragma mark === Accessor Methods ===
//	*********************************************************************************************************************************
//	archive strings
	@synthesize pfFileNameStr;
//	slider values
	@synthesize amplitudeFlt;

	#pragma mark -
	#pragma mark === Internal Methods ===
//	*********************************************************************************************************************************
- (id)init
{
    self = [super init];
    if (self) {
	//	pfPlayFileModelPtr = [[AQPlayFileModel2 alloc] init];
		pfPlayFileModelPtr = [AQPlayFileModel2 new];
	//
		pfFileNameStr = @"";
		
		NSNotificationCenter *	nc = [NSNotificationCenter defaultCenter];
		[nc addObserver:self selector:@selector(pfPlayBackStopNotification:) name:@"PlayBackStop" object:nil];
		
		[self setAmplitudeFlt:0.75];
		
	//	key value observing
		[self addObserver:self forKeyPath:@"amplitudeFlt" options:NSKeyValueObservingOptionOld context:nil];
    }
    
    return self;
}

- (void)dealloc
{
    [super dealloc];
}

	#pragma mark -
	#pragma mark === Key Value Observing Methods ===
//	*********************************************************************************************************************************
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context
{
//	NSLog(@"observeValueForKeyPath");
	if([keyPath compare:@"amplitudeFlt"] == NSOrderedSame) {
		[pfPlayFileModelPtr fpSetamplitude:[self amplitudeFlt]];
	}
}

	#pragma mark -
	#pragma mark === Notification Methods ===
//	*********************************************************************************************************************************
- (void)pfPlayBackStopNotification:(NSNotification *)notification {
	NSLog(@"pfPlayBackStopNotification Start");
	BOOL succes = [pfPlayFileModelPtr pfStopPlayBack];
	if(succes == YES) {
		[pfBrowse  setEnabled:YES];
		[pfStartPlayBack setEnabled:NO];
		[pfStopPlayBack setEnabled:NO];
	} else {
		NSLog(@"Stop Recording failed");
	}
}

	#pragma mark -
	#pragma mark === Action Methods ===
//	*********************************************************************************************************************************
- (IBAction)pfReadButtons:(id)sender {
	NSLog(@"pfReadButtons Started");
	BOOL	succes = YES;
	
	switch([sender tag])
	{
		case 0:		//	browse
			[self pfOpenSoundFile];
			break;
		
		case 1:		//	start playback
			NSLog(@"pfFilePathStr: %@", pfFileNameStr);
			NSURL *	soundFileURL = [NSURL fileURLWithPath:pfFileNameStr];
			succes = [pfPlayFileModelPtr pfConfigureAudioFile2:(CFURLRef)soundFileURL];
			if(succes == NO) {
				NSLog(@"ConfigureAudioFile failed");
			}
			succes = [pfPlayFileModelPtr pfStartPlayBack];
			if(succes == YES) {
			[pfStartPlayBack setEnabled:NO];
			[pfStopPlayBack setEnabled:YES];
			} else {
			NSLog(@"Start Recording failed");
			}
			break;
		
		case 2:		//	stop playback
			succes = [pfPlayFileModelPtr pfStopPlayBack];
			if(succes == YES) {
				[pfBrowse  setEnabled:YES];
				[pfStartPlayBack setEnabled:NO];
				[pfStopPlayBack setEnabled:NO];
			} else {
				NSLog(@"Stop Recording failed");
			}
			break;
	}
}

	#pragma mark -
	#pragma mark === Data Archive Methods ===
//	*********************************************************************************************************************************
- (void)pfOpenSoundFile {
    NSOpenPanel*	openPanel = [NSOpenPanel openPanel];
	NSArray *		fileTypes = [NSArray arrayWithObjects:@"m4a", @"mp3", @"aiff", nil];
	
	[openPanel setAllowedFileTypes:fileTypes];
    [openPanel setAllowsMultipleSelection:NO];
	[openPanel beginSheetModalForWindow:pfWindow completionHandler:^(NSInteger result) {
        if (result == NSOKButton) {
            [openPanel orderOut:self];		//	close panel before we might present an error
			[pfBrowse  setEnabled:NO];
			[pfStartPlayBack setEnabled:YES];
			NSArray *	filesToOpen = [openPanel filenames];
			[self setPfFileNameStr:[filesToOpen objectAtIndex:0]];
			NSLog(@"Open Panel Returned: %@.\n", pfFileNameStr);
        }
    }];
}

@end
